#lang racket

; Partially apply a function of two parameters to the given value.
(define partial2to1
   (lambda (f x)             ; this x is the closure for the nested lambda
      (lambda (y) (f x y)))) ; x is a non-local reference in this lambda

; Partially apply a function of three parameters to the given value.
(define partial3to2
   (lambda (f x)
      (lambda (y z) (f x y z))))

; Partially apply a function of four parameters to the given value.
(define partial4to3
   (lambda (f x)
      (lambda (y z w) (f x y z w))))

; Partially apply a function of three parameters to the given two values.
(define partial3to1
   (lambda (f x y)
      (lambda (z) (f x y z))))

; Return a curried version of a function of two parameters.
(define curried2
   (lambda (f)
      (lambda (x)
         (lambda (y) (f x y)))))

; Return a curried version of a function of three parameters.
(define curried3
   (lambda (f)
      (lambda (x)
         (curried2 (partial3to2 f x)))))

; Return a curried version of a function of four parameters.
(define curried4
   (lambda (f)
      (lambda (x)
         (curried3 (partial4to3 f x)))))

(define f (lambda (x y z w) (+ x y z w)))
(define fc (curried4 f))
(define f7 (fc 7))
(define f75 (f7 5))
((f75 1) 2)   ; (f 7 5 1 2) ==> 15
